--=======================================================================================================
--  REGISTER ROCK LOADING AUTO PICK-UP SPECIALIZATION
-- 
-- @Author:    	Xentro (Marcus@Xentro.se)
-- @Website:	www.Xentro.se
-- @History:	v2.0  - 2017-05-05 - 
-- 				v1.53 - 2014-11-11 - Improvement
-- 				v1.52 - 2013-10-29 - 
--
-- @Edit:		akuenzi
-- @Purpose:	Add specialization to trailers, enabling them to auto pick up rocks.
-- @Date:		12/26/2018
--=======================================================================================================


AddSpecialization1 = {};
AddSpecialization1.proceedWithLoading = false;
AddSpecialization1.g_currentModDirectory = g_currentModDirectory;

addModEventListener(AddSpecialization1);

-- Insert the RockSpawn specialization into vehicles with either of the following specializations.
local RockLoadSpecializations = {
				-- Add specialization to trailers...
				{"trailer", true},
				
				--[[But NOT to trailers that also have the following specializations.
				{"strawBlower", false},
				{"mixerWagon", false},
				{"hookLiftTrailer", false},
				{"forageWagon", false},
				{"waterTrailer", false},
				{"overloading", false},
				{"mower", false},]]
								}
-- Configuration
local specializationToAdd1 = {};

for i = 1, table.getn(RockLoadSpecializations) do
	local entry = {};
	entry.specialization  	= "RockLoad"; 			-- Name of specialization
	entry.class1Name  		= "RockLoad"; 			-- Class1 name of specialization
	entry.filename  		= "Scripts/RockLoad"; 	-- file name of specialization
	entry.l10nNameTag		= "LEVEL"; 				-- Make text global so we can use it on the mod. Name it like following in modDesc GAME_EXTENSION_01, GAME_EXTENSION_02... 
	entry.vehicleTypeLimit 	= { 					-- Limit to what vehicle type to add specialization to, will speed up the process1 big time but its only optional, can be left empty.
		-- VehicleType, ALLOW
		-- tractor = true
	};
	entry.specLimit 	  	= { 								-- Limit adding to must have specialization or cant have specialization - Only global scripts (default scripts or global mods)
		{RockLoadSpecializations[i][1], RockLoadSpecializations[i][2]},
		--print("Specialization: " .. RockLoadSpecializations[i][1] .. " - " .. tostring(RockLoadSpecializations[i][2]))
		-- NAME, REQUIRED / CANT HAVE (True / False)
		-- {"drivable", true} 					
	};
	entry.searchWords  		= {									-- Limit adding to must have specialization or cant have specialization - This will only check custom specialization's in the vehicleType!
		-- NAME, REQUIRED / CANT HAVE (True / False)
		-- {"somethingOkay" , false}
	};

	table.insert(specializationToAdd1, entry);
end;

for i, ss1 in ipairs(specializationToAdd1) do
	table.insert(ss1.specLimit,   {ss1.specialization, false});
	table.insert(ss1.searchWords, {ss1.specialization, false});
	
	if SpecializationUtil.specializations[ss1.specialization] == nil then
		SpecializationUtil.registerSpecialization(ss1.specialization, ss1.class1Name, g_currentModDirectory .. ss1.filename .. ".lua");
		AddSpecialization1.proceedWithLoading = true;
	else
		print("Add Specialization (" .. ss1.specialization .. ") - Error: Specialization have been loaded already by another mod! This process1 will stop now.");
		table.remove(specializationToAdd1, i);
	end;
end;

function AddSpecialization1:loadMap(name)	
	-- This is only called once for FS17 so no additional code needed as with older versions.
	if AddSpecialization1.proceedWithLoading then
		AddSpecialization1:add();
	end;
end;

function AddSpecialization1:deleteMap()
end;

function AddSpecialization1:mouseEvent(posX, posY, isDown, isUp, button)
end;

function AddSpecialization1:keyEvent(unicode, sym, modifier, isDown)
end;

function AddSpecialization1:update(dt)
end;

function AddSpecialization1:draw()
end;

function AddSpecialization1:add()
	for _, ss1 in ipairs(specializationToAdd1) do
		local printHeader = "Add Specialization (" .. ss1.specialization .. ") - Info: ";
		local vtIsEmpty = true;
		local vtLetNoneFalsePass1 = false;
		
		for name, v in pairs(ss1.vehicleTypeLimit) do
			vtIsEmpty = false;
			
			if not v then
				vtLetNoneFalsePass1 = true; -- If only false then let all types which havent been setup pass1.
			else
				vtLetNoneFalsePass1 = false; -- If we got both True/False then only allow those that are set to true.
				break;
			end;
		end;
		
		for vehicleType, vehicle in pairs(VehicleTypeUtil.vehicleTypes) do
			if vehicle ~= nil then
				local vt = Utils.splitString(".", vehicleType);
				local vtCheck = vehicleType;
				
				-- We are an mod!
				if #vt > 1 then
					vtCheck = vt[2];
				end;
				
				-- VehicleType search is empty, let it pass1
				if (vtIsEmpty 
				-- Check True / False state and let pass1 according
				or not vtLetNoneFalsePass1 and ss1.vehicleTypeLimit[vtCheck] ~= nil and ss1.vehicleTypeLimit[vtCheck]
				-- Only false vehicleType's have been set, let all which arent set to False pass1
				or vtLetNoneFalsePass1 and ss1.vehicleTypeLimit[vtCheck] ~= nil and ss1.vehicleTypeLimit[vtCheck]
				or vtLetNoneFalsePass1 and ss1.vehicleTypeLimit[vtCheck] == nil) then
					local forceStop = false;
					local numLimit = {0, 0}; -- Required global scripts found, Must have x specs 
					local errorHelp = {};
					
					-- Global script check
					for _, sl in ipairs(ss1.specLimit) do
						if sl[2] then
							numLimit[2] = numLimit[2] + 1;
						end;
						
						for _, vs in ipairs(vehicle.specializations) do
							if vs == SpecializationUtil.getSpecialization(sl[1]) then
								if sl[2] then
									errorHelp[sl[1]] = true;
									numLimit[1] = numLimit[1] + 1;
								else
									--print(printHeader .. "Unallowed specialization (" .. sl[1] .. ") was found in " .. vehicleType);
									forceStop = true;
									break;
								end;
							end;
						end;
						
						if forceStop then
							break;
						end;
					end;
					
					-- Do name check of custom specializations
					if not forceStop then
						local specializationToSearch = {};
						
						-- Gather all scripts used for vehicle, none global.
						for name in pairs(SpecializationUtil.specializations) do
							if string.find(name, vt[1]) ~= nil then
								local parts = Utils.splitString(".", name);
								
								if #parts > 1 then
									table.insert(specializationToSearch, parts);
								end;
							end;
						end;
						
						-- Process1 the result 
						for _, sw in ipairs(ss1.searchWords) do
							if sw[2] then
								numLimit[2] = numLimit[2] + 1;
							end;
							
							for _, s in ipairs(specializationToSearch) do
								local stringMatch = string.find(string.lower(s[2]), string.lower(sw[1])) ~= nil;
								
								if stringMatch then
									if not sw[2] then
										--print(printHeader .. "Unallowed specialization (" .. s[2] .. ") was found in " .. vehicleType);
										forceStop = true;
										break;
									else
										-- We need to count to make sure all required scripts are found.
										numLimit[1] = numLimit[1] + 1;
										errorHelp[sw[1]] = s[2];
									end;
								end;
							end;
							
							if forceStop then
								break;
							end;
						end;
					end;
					
					-- Check if all required scripts was found.
					if numLimit[1] ~= numLimit[2] then
						--print(printHeader .. "Not all required specialization's was found (" .. numLimit[1] .. "/" .. numLimit[2] .. ") in " .. vehicleType);
						--print(printHeader .. "List of required specialization's");
						
						for _, sl in ipairs(ss1.specLimit) do
							if sl[2] then
								--print(printHeader .. sl[1] .. " was found: " .. tostring(errorHelp[sl[1]] ~= nil));
							end;
						end;
						
						for _, sw in ipairs(ss1.searchWords) do
							if sw[2] then
								--print(printHeader .. sw[1] .. " was found: " .. tostring(errorHelp[sw[1]] ~= nil) .. " (" .. tostring(errorHelp[sw[1]]) ..")");
							end;
						end;
						
						forceStop = true;
					end;
					
					-- Finish, add script.
					if not forceStop then
						table.insert(vehicle.specializations, SpecializationUtil.getSpecialization(ss1.specialization));
						--print(printHeader .. "Inserted on " .. vehicleType);
					end;
				end;
			end;
		end;
		
		-- make l10n global 
		-- None to make global with 'LEVEL' in them at this time.
		--[[local i = 0;
		while true do
			local txt = string.format(ss1.l10nNameTag .. "_%d", i);
			
			if not g_i18n:hasText(txt) then
				break;
			end;
			
			g_i18n.globalI18N.texts[txt] = g_i18n:getText(txt);
			
			i = i + 1;
		end;]]
		
		-- Added code to make l10n global.
		g_i18n.globalI18N.texts["input_RockLoadStatus"] = g_i18n:getText("input_RockLoadStatus");
		g_i18n.globalI18N.texts["input_RockUnLoadingStatus"] = g_i18n:getText("input_RockUnLoadingStatus");
		g_i18n.globalI18N.texts["ROCK_LOAD_STATUS_TURN_OFF"] = g_i18n:getText("ROCK_LOAD_STATUS_TURN_OFF");
		g_i18n.globalI18N.texts["ROCK_LOAD_STATUS_TURN_ON"] = g_i18n:getText("ROCK_LOAD_STATUS_TURN_ON");
		g_i18n.globalI18N.texts["input_RockAreaShow"] = g_i18n:getText("input_RockAreaShow");
		
	end;
end;